/*---------------------------------------------------------------------------\
| 18F458 DEMO SOFTWARE
|
| File:                 Timer2Module.c          All Timer2 Code
|
| Global Routines:      InitializeTimer2()
|                       Timer2Isr()
|
| Local Routines:       None
|
| Discussion:           Sure.
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 02/10/2002 | Created. Again.
\---------------------------------------------------------------------------*/


//------------------------------------------------------------------------
//  #includes and local #defines
//------------------------------------------------------------------------

#include "core.h"   // Includes for the project (includes other includes!)


//------------------------------------------------------------------------
//  Global Variables
//------------------------------------------------------------------------

// None

//------------------------------------------------------------------------
//  Module-wide variables
//------------------------------------------------------------------------

// None


/*---------------------------------------------------------------------------\
| Function:     InitializeTimer2
|
| Purpose:      Guess.
|
| Discussion:   Huh?
|
| Expects:      All interupts to be off (after all, we were just reset).
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 05/21/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

void InitializeTimer2 (enum TIMER2_INIT_MASK optionsMask)
{

    // On reset, Timer2 is set to the following RESET values:
    //
    // T2CON = 0b00100101
    //           ||||||||
    //           |||||||+----------- T2CKPS0 = 00 = prescaler 1:1
    //           ||||||+------------ T2CKPS1
    //           |||||+------------- TMR2ON  = 0 = Timer2 off
    //           ||||+-------------- TOUTPS0 = 0000 = postscaler 1:1
    //           |||+--------------- TOUTPS1
    //           ||+---------------- TOUTPS2
    //           |+----------------- TOUTPS3
    //           +------------------ <Unimplemented on PIC>
    //
    //  PR2 = 0xFF

    if (optionsMask & TIMER2_ENABLE)
    {
        switch (optionsMask & 0b00000011)
        {

            case TIMER2_250HZ:
                PR2 = 249;              // ASSUMES FOSC = 20MHZ
                T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
                break;

            case TIMER2_500HZ:
                PR2 = 249;              // ASSUMES FOSC = 20MHZ
                T2CON |= 0b01010101;    // ASSUMES FOSC = 20MHZ
                break;

            case TIMER2_1KHZ:
                PR2 = 249;              // ASSUMES FOSC = 20MHZ
                T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
                break;

            case TIMER2_2KHZ:
                PR2 = 124;              // ASSUMES FOSC = 20MHZ
                T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
                break;

            case TIMER2_2_5KHZ:
                PR2 = 24;               // ASSUMES FOSC = 20MHZ
                T2CON |= 0b00101110;    // ASSUMES FOSC = 20MHZ
                break;

            case TIMER2_5KHZ:
                PR2 = 249;              // ASSUMES FOSC = 20MHZ
                T2CON |= 0b00000101;    // ASSUMES FOSC = 20MHZ
                break;

            default:                    // Default: 1KHz
                PR2 = 249;              // ASSUMES FOSC = 20MHZ
                T2CON |= 0b00101101;    // ASSUMES FOSC = 20MHZ
                ThrowWarning(WARN_TIMER2_RATE);
                break;
        }

        if(optionsMask & TIMER2_ENABLE_INTS)
            EnableTimer2Ints();
        else
            DisableTimer2Ints();
    }
}


/*---------------------------------------------------------------------------\
| Function:     Timer2Isr
|
| Purpose:      Handle the Timer 2 interrupt.
|
| Discussion:   Clears the Timer2 interrupt and if an interrupt handler exists
|               (the function pointer is non-null) then it calls the handler.
|               Note that if the Timer 2 interrupts are turned off, this
|               function goes away (and thus never has the opportunity to
|               vector off to la-la land.
|
| Expects:      -
|
|    Date    | Notes
|------------|------------------------------------------------------------
| 08/10/2002 | Added application specific interrup handler via function ptr.
| 02/10/2002 | Created if you can call it that.
\---------------------------------------------------------------------------*/

#if INTS_TIMER2_ISR_USED == INTS_CORE_ISR_ENABLED

void Timer2Isr (void)
{
    ClearTimer2Interrupt();

    // If an application specific interrupt handler is defined,
    // then call it. See init_int.h.

    #if INTS_TIMER2_APPLICATION_HOOK == INTS_HOOK_ENABLED
    Timer2ApplicationHandler();
    #endif
}

#endif

